// Mandelbrot Mathlink Client
// Original CUDA program submitted by Mark Granger, NewTek
// Client was written by Abdulmajed Dakkak
// Mathlink interface was written by Abdulmajed Dakkak

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include "mathlink.h"

#define INVALIDATE_STEP 3

#define GETNEWFRAME 0x0
#define ZOOMIN      0x2
#define ZOOMOUT     0x3
#define MOVERIGHT   0x4
#define MOVEUP      0x5
#define MOVELEFT    0x6
#define MOVEDOWN    0x7
#define TERMINATE   0x8
#define MOUSE       0x9
#define CLICK       0x1
#define MOTION      0x2
#define LEFTCLICK   0x0
#define RIGHTCLICK  0x1

//Original image width and height
int imageW, imageH, max_pass;

// Starting position and scale
double xOff = -0.5;
double yOff = 0.0;
double scale = 3.2;

// Starting stationary position and scale motion
double xdOff = 0.0;
double ydOff = 0.0;

// SOCKET ID
int sockfd;

// Request packet
char request[16];

// D_DST Def
char * d_dst;

short invalid_frame = 1;

/**********Network Functions************/
int connectToServer(int port, char* IP) {
   fprintf(stderr,"Attempting to connect to server...\n");
   struct sockaddr_in remote_addr;

   sockfd=socket(PF_INET,SOCK_STREAM,0);
   
   remote_addr.sin_family=AF_INET;
   remote_addr.sin_port=htons(port);
   remote_addr.sin_addr.s_addr=inet_addr(IP);
   memset(remote_addr.sin_zero, '\0',sizeof remote_addr.sin_zero);

   if(connect(sockfd,(struct sockaddr*)&remote_addr,sizeof remote_addr)==-1){
      perror("connect");
      exit(1);
   }
   
   fprintf(stderr,"Connected to server.\n");
   return 1;
}

int sendEnvironmentVariables( ) {
    int env[3] = {imageW, imageH, max_pass};
    send(sockfd, env, 3 * sizeof(int), 0);
    return 1;
}

int * getNewFrame( ) {
//   fprintf(stderr,"Requesting new frame...\n");
   int i;

   request[0]= GETNEWFRAME;
   send(sockfd, request, 16 * sizeof(char), 0);

   int bytesRecieved=recv(sockfd, d_dst, 
                          imageW * imageH * 4 * sizeof(char), MSG_WAITALL);
   if(bytesRecieved==-1) { perror("recv()"); exit(1); }

   int data[imageW * imageH * 4];
   for(i = 0; i < imageW * imageH * 4; i++) {
       data[i] = *(d_dst + i);
   }
   MLPutIntegerList(stdlink, (int *) data, imageW * imageH * 4);
//   fprintf(stderr,"Received frame. (%d bytes).\n",bytesRecieved);
   return ;
}

int sendRequestToZoomIn( ) {
    request[0] = ZOOMIN;
    send(sockfd, request, 16 * sizeof(char), 0);
    return 1;
}

int sendRequestToZoomOut( ) {
    request[0] = ZOOMOUT;
    send(sockfd, request, 16 * sizeof(char), 0);
    return 1;
}

int sendRequestToMoveLeft( ) {
    request[0] = MOVELEFT;
    send(sockfd, request, 16 * sizeof(char), 0);
    return 1;
}

int sendRequestToMoveUp( ) {
    request[0] = MOVEUP;
    send(sockfd, request, 16 * sizeof(char), 0);
    return 1;
}

int sendRequestToMoveRight( ) {
    request[0] = MOVERIGHT;
    send(sockfd, request, 16 * sizeof(char), 0);
    return 1;
}

int sendRequestToMoveDown( ) {
    request[0] = MOVEDOWN;
    send(sockfd, request, 16 * sizeof(char), 0);
    return 1;
}

int sendRequestToTerminate( ) {
    request[0] = TERMINATE;
    send(sockfd, request, 16 * sizeof(char), 0);
    return 1;
}

int sendClickEvent(int button_clicked, int x, int y) {
    short button;
    if(button_clicked == 0) {
        button = LEFTCLICK;
    }
    else {
        button = RIGHTCLICK;
    }

    request[0] = MOUSE;
    request[1] = CLICK;
    request[2] = button;
    int x_y[2] = {x, y};
    memcpy(&request[3], x_y, 2*sizeof(int));
//    printf("Sent Click event %d, %d, %d\n", button, x_y[0], x_y[1]);
    send(sockfd, (void*)request, 16 * sizeof(char), 0);
    return 1;
}

int sendMotionEvent(int x, int y) {
    request[0] = MOUSE;
    request[1] = MOTION;

    int x_y[2] = {x, y};
    memcpy(&request[2], x_y, 2*sizeof(int));

    send(sockfd, request, 16 * sizeof(char), 0);
    return 1;
}

/*****END**Network Functions**END*******/

int setEnvironmentVariables(int image_width, int image_height, 
        int iterations_per_frame) {
	imageW = image_width;
	imageH = image_height;
    max_pass = iterations_per_frame;
    d_dst = (char *) malloc(image_width * image_height * 4);
    sendEnvironmentVariables( );
    return 1;
}

#if MACINTOSH_MATHLINK

int main( int argc, char* argv[])
{
	/* Due to a bug in some standard C libraries that have shipped with
	 * MPW, zero is passed to MLMain below.  (If you build this program
	 * as an MPW tool, you can change the zero to argc.)
	 */
	argc = argc; /* suppress warning */
	return MLMain( 0, argv);
}

#elif WINDOWS_MATHLINK

#if __BORLANDC__
#pragma argsused
#endif

int PASCAL WinMain( HINSTANCE hinstCurrent, HINSTANCE hinstPrevious, LPSTR lpszCmdLine, int nCmdShow)
{
	char  buff[512];
	char FAR * buff_start = buff;
	char FAR * argv[32];
	char FAR * FAR * argv_end = argv + 32;

	hinstPrevious = hinstPrevious; /* suppress warning */

	if( !MLInitializeIcon( hinstCurrent, nCmdShow)) return 1;
	MLScanString( argv, &argv_end, &lpszCmdLine, &buff_start);
	return MLMain( argv_end - argv, argv);
}

#else

int main(int argc, char* argv[])
{
	return MLMain(argc, argv);
}

#endif
